@@ -1,23 +1,25 @@
 /* 
  * driver.c
+ * 
  * Copyright (C) 2002, AVM GmbH. All rights reserved.
  * 
- * This Software is  free software. You can redistribute and/or
- * modify such free software under the terms of the GNU Lesser General Public
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
  * 
- * The free software is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  * 
  * You should have received a copy of the GNU Lesser General Public
- * License along with this Software; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA, or see
- * http://www.opensource.org/licenses/lgpl-license.html
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301  USA
  * 
- * Contact: AVM GmbH, Alt-Moabit 95, 10559 Berlin, Germany, email: info@avm.de
+ * Contact:
+ *   AVM GmbH, Alt-Moabit 95, 10559 Berlin, Germany, email: info@avm.de
  */
 
 #include <asm/atomic.h>
@@ -96,8 +98,13 @@ static DECLARE_WAIT_QUEUE_HEAD(wait);
 static DECLARE_WAIT_QUEUE_HEAD(capi_wait);
 static DECLARE_WAIT_QUEUE_HEAD(tx_wait);
 static DECLARE_WAIT_QUEUE_HEAD(rx_wait);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,23)
+static DECLARE_COMPLETION(hotplug);
+static DECLARE_COMPLETION(notify);
+#else
 static DECLARE_MUTEX_LOCKED(hotplug);
 static DECLARE_MUTEX_LOCKED(notify);
+#endif
 
 #define	SCHED_WAKEUP_CAPI	{ atomic_set (&thread_capi_flag, 1); wake_up_interruptible (&capi_wait); }
 #define	SCHED_WAKEUP		{ atomic_set (&got_kicked, 1); wake_up_interruptible (&wait); }
@@ -1245,7 +1252,11 @@ static int scheduler (void * arg) {
 		}
 	}
 	LOG("Scheduler thread stopped.\n");
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,23)
+	complete (&hotplug);
+#else
 	up (&hotplug);
+#endif
 	return 0;
 } /* scheduler */
 
@@ -1271,7 +1282,11 @@ void kill_thread (void) {
 
 	if (thread_pid != -1) {
 		atomic_set (&thread_run_flag, 0);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,25)  
+		if (NULL == find_task_by_vpid (thread_pid)) {
+#else
 		if (NULL == find_task_by_pid (thread_pid)) {
+#endif
 			LOG("Thread[%d] has died before!\n", thread_pid);
 		} else {
 			if (!atomic_read (&thread_capi_flag)) {
@@ -1280,7 +1295,11 @@ void kill_thread (void) {
 				SCHED_WAKEUP;
 			}
 			LOG("Scheduler thread signalled, waiting...\n");
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,23)
+			wait_for_completion (&hotplug);
+#else
 			down (&hotplug);
+#endif
 			LOG("Scheduler thread[%d] terminated.\n", thread_pid);
 		}
 		thread_pid = -1;
